from typing import Tuple, Dict, Optional
import numpy as np
import pygame
import pygame_gui
from pygame_gui import UIManager
from pygame_gui.core import IContainerLikeInterface
from pygame_gui.elements import UITextBox, UIImage

# Constants
DEFAULT_FONT = 'Arial'
DEFAULT_FONT_SIZE = 30
DEFAULT_COLOR = (0, 0, 0)
DEFAULT_BG_COLOR = None
DEFAULT_ANCHOR = 'center'


def gradient_surface(rect: pygame.Rect,
                     start_color: Tuple[int, int, int],
                     end_color: Tuple[int, int, int]) -> pygame.Surface:
    surface = pygame.Surface(rect.size, pygame.SRCALPHA)
    for x in range(rect.w):
        r = start_color[0] + (end_color[0] - start_color[0]) * (x / rect.w)
        g = start_color[1] + (end_color[1] - start_color[1]) * (x / rect.w)
        b = start_color[2] + (end_color[2] - start_color[2]) * (x / rect.w)
        pygame.draw.line(surface, (r, g, b), (x, 0), (x, rect.h))
    return surface


def rounded_gradient_surface(rect: pygame.Rect,
                             start_color: Tuple[int, int, int],
                             end_color: Tuple[int, int, int],
                             corner_radius: int,
                             edge_thickness: int = 1,
                             edge_color: Tuple[int, int, int] = (0, 0, 0)) -> pygame.Surface:
    surface = gradient_surface(rect, start_color, end_color)
    mask = pygame.Surface(rect.size, pygame.SRCALPHA)
    pygame.draw.rect(mask, (255, 255, 255), rect, border_radius=corner_radius)
    surface.blit(mask, (0, 0), special_flags=pygame.BLEND_RGBA_MIN)
    pygame.draw.rect(surface, edge_color, rect, width=edge_thickness, border_radius=corner_radius)
    return surface


class PG_Text:
    def __init__(self, text='', xy=(0, 0),
                 color: Tuple[int, int, int] = DEFAULT_COLOR,
                 bg_color: Optional[Tuple[int, int, int]] = DEFAULT_BG_COLOR,
                 font_name: str = DEFAULT_FONT,
                 font_size: int = DEFAULT_FONT_SIZE,
                 font: Optional[pygame.font.Font] = None,
                 anchor: str = DEFAULT_ANCHOR):
        self.text = text
        self.xy = xy
        self.color = color
        self.bg_color = bg_color
        self.font_name = font_name
        self.font_size = font_size
        self.font = font
        self.anchor = anchor
        self.update_text()

    def __str__(self):
        return f'{self.text}, <color({self.color} {self.bg_color})>, <font({self.font}, {self.font_name}, {self.font_size})>'

    def update_text(self):
        font = self.font or pygame.font.SysFont(self.font_name, self.font_size)
        self.text_surface = font.render(self.text, True, self.color, self.bg_color)
        self.text_rect = self.text_surface.get_rect()
        setattr(self.text_rect, self.anchor, self.xy)

    def draw(self, surface: pygame.Surface):
        surface.blit(self.text_surface, self.text_rect)


class TextBoxSurface:
    def __init__(self, rect: pygame.Rect,
                 manager: Optional[UIManager] = None,
                 container=None):
        self.rect = rect
        self.manager = manager
        self.container = container
        self.texts: Dict[str, PG_Text] = {}
        self.surface = pygame.Surface(rect.size, pygame.SRCALPHA)

        self.text_box = UITextBox(
            html_text="",
            relative_rect=rect,
            manager=self.manager,
            container=self.container
        )

        self.image_element = UIImage(
            relative_rect=self.rect,
            image_surface=self.surface,
            manager=manager,
            container=container
        )

        self.clear()

    def clear(self):
        self.surface.fill((0, 0, 0, 0))

    def add_text(self, name: str, text='',
                 xy: Optional[Tuple[int, int]] = None,
                 color: Tuple[int, int, int] = DEFAULT_COLOR,
                 bg_color: Optional[Tuple[int, int, int]] = DEFAULT_BG_COLOR,
                 font_name: str = DEFAULT_FONT,
                 font_size: int = DEFAULT_FONT_SIZE,
                 font: Optional[pygame.font.Font] = None,
                 anchor: str = DEFAULT_ANCHOR):
        xy = xy or tuple(np.array(self.rect.size) / 2)
        self.texts[name] = PG_Text(text, xy, color, bg_color, font_name, font_size, font, anchor)
        self.set_image()

    def update_text(self, name: str, **kwargs):
        if name not in self.texts:
            self.add_text(name)

        text_obj = self.texts[name]
        for k, v in kwargs.items():
            setattr(text_obj, k, v)
        text_obj.update_text()
        self.set_image()

    def set_image(self):
        self.clear()
        for text_obj in self.texts.values():
            text_obj.draw(self.surface)
        self.image_element.set_image(self.surface)

    def set_background_text(self, html_text: str):
        self.text_box.set_text(html_text)


def main():
    pygame.init()
    window_size = (800, 600)
    display = pygame.display.set_mode(window_size)
    pygame.display.set_caption('UIImage Example')
    manager = UIManager(window_size)

    textbox1 = TextBoxSurface(pygame.Rect(100, 100, 300, 100), manager=manager)
    textbox1.set_background_text("This is background text")
    textbox1.update_text(
        'Text', text='Hello World', color=(255, 0, 255), xy=(10, 20), anchor='topleft',
        font_name='Arial', font_size=10
    )
    textbox1.update_text(
        'Text2', text='abc', color=(255, 0, 255), xy=(10, 50), anchor='topleft',
        font_name='Rounded Mplus 1c Medium', font_size=20
    )

    windows = pygame_gui.elements.UIWindow(pygame.Rect(100, 200, 600, 500), manager=manager)
    textbox2 = TextBoxSurface(pygame.Rect(100, 100, 300, 100), manager=manager, container=windows)
    textbox2.set_background_text("Window background text")
    textbox2.update_text('Text', text='My World', color=(255, 0, 255))

    clock = pygame.time.Clock()
    is_running = True

    while is_running:
        time_delta = clock.tick(60) / 1000.0

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                is_running = False
            elif event.type == pygame.MOUSEBUTTONDOWN:
                textbox1.update_text('Text', text='Hello')

            manager.process_events(event)

        manager.update(time_delta)

        display.fill((255, 255, 255))
        manager.draw_ui(display)

        pygame.display.update()

    pygame.quit()


if __name__ == '__main__':
    main()



import pygame
import pygame_gui
import warnings


def putText(surface, text, xy, font_size=32, color=(0, 0, 0), bg_color=None, anchor='center', font=None):
    font_ = pygame.font.Font(font, font_size)
    text_surface = font_.render(text, True, color, bg_color, wraplength=0)
    text_rect = text_surface.get_rect()
    setattr(text_rect, anchor, xy)
    surface.blit(text_surface, text_rect)


''' font size upgrade '''

warnings.filterwarnings("ignore", category=UserWarning, module="pygame_gui.core.ui_font_dictionary")
warnings.filterwarnings("ignore", category=UserWarning, module="pygame_gui.core.text.html_parser")


class HTMLParser(pygame_gui.core.text.html_parser.HTMLParser):
    # font_sizes = {
    #     1: 8,
    #     1.5: 9,
    #     2: 10,
    #     2.5: 11,
    #     3: 12,
    #     3.5: 13,
    #     4: 14,
    #     4.5: 16,
    #     5: 18,
    #     5.5: 20,
    #     6: 24,
    #     6.5: 32,
    #     7: 48,
    #     8: 200  # add font data
    # }

    # or

    def _handle_font_tag(self, attributes, style):
        super()._handle_font_tag(attributes, style)
        if 'size' in attributes:
            if attributes['size'] is not None and len(attributes['size']) > 0:
                try:
                    font_size = int(attributes['size'])
                except ValueError or AttributeError:
                    font_size = self.default_style['font_size']
            else:
                font_size = self.default_style['font_size']
            style["font_size"] = font_size


class UITextBox(pygame_gui.elements.UITextBox):
    def _reparse_and_rebuild(self):
        self.parser = HTMLParser(self.ui_theme, self.combined_element_ids,
                                 self.link_style, line_spacing=self.line_spacing,
                                 text_direction=self.font_dict.get_default_font().get_direction())
        self.rebuild()



import time
import hexss
from hexss.raspberrypi.gpio import DigitalInputDevice, DigitalOutputDevice

hexss.check_packages(
    'requests',
    auto_install=True
)
import requests


class GPIOMapping:
    def __init__(self):
        self.rad_tower = DigitalOutputDevice(4, active_high=True)
        self.buzzer = DigitalOutputDevice(17, active_high=True)
        self.yello_tower = DigitalOutputDevice(18, active_high=True)
        self.yello_LED = DigitalOutputDevice(27, active_high=True)
        self.green_tower = DigitalOutputDevice(22, active_high=True)
        self.green_LED = DigitalOutputDevice(23, active_high=True)
        self.o7 = DigitalOutputDevice(24, active_high=True)
        self.o8 = DigitalOutputDevice(25, active_high=True)

        self.i1 = DigitalInputDevice(5, bounce_time=0.1)
        self.yello_button = DigitalInputDevice(6, bounce_time=0.1)
        self.i3 = DigitalInputDevice(12, bounce_time=0.1)
        self.green_button = DigitalInputDevice(13, bounce_time=0.1)
        self.i5 = DigitalInputDevice(16, bounce_time=0.1)
        self.i6 = DigitalInputDevice(19, bounce_time=0.1)
        self.i7 = DigitalInputDevice(20, bounce_time=0.1)
        self.i8 = DigitalInputDevice(21, bounce_time=0.1)

        self._prev_green = self.green_button.value
        self.green_LED.on()

    def update(self):
        url = 'http://192.168.225.137:3000'
        current = self.green_button.value
        if self._prev_green == 0 and current == 1:  # rising edge
            try:
                r = requests.post(url, data={'button': 'Capture&Predict'})
                print("RISING detected:", r.status_code)
                self.green_LED.off()
            except Exception as e:
                print(e)
        self._prev_green = current

        try:
            r = requests.get(f'{url}/data')
            data = r.json()
            res = data['config'].get('res')
            if res == 'OK':
                self.rad_tower.off()
                self.yello_tower.off()
                self.green_tower.on()

                self.green_LED.on()
            elif res == 'NG':
                self.rad_tower.on()
                self.yello_tower.off()
                self.green_tower.off()

                self.green_LED.on()
            else:
                self.rad_tower.off()
                self.yello_tower.on()
                self.green_tower.off()

        except Exception as e:
            print(e)


if __name__ == "__main__":
    gpio = GPIOMapping()

    while True:
        gpio.update()
        time.sleep(0.1)



import os
import random
import shutil
import json
import pathlib
from datetime import datetime
from hexss import json_load, json_update
from hexss.constants import *
from hexss.image import controller, crop_img

import cv2
import matplotlib.pyplot as plt
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers, models
from tensorflow.keras.models import Sequential


def save_img(config: dict, model_name: str, frame_dict: dict, paths: dict, resize: tuple = (180, 180)) -> None:
    img_full_path = pathlib.Path(paths['img_full'])
    img_frame_path = pathlib.Path(paths['img_frame'])
    img_frame_log_path = pathlib.Path(paths['img_frame_log'])

    # Remove existing folders for this model, if any
    for base_path in [img_frame_log_path, img_frame_path]:
        folder = base_path / model_name
        if folder.exists():
            shutil.rmtree(folder)

    # Get list of image files
    img_files = sorted({f.stem for f in img_full_path.glob("*") if f.suffix in ['.png', '.json']}, reverse=True)

    for i, file_name in enumerate(img_files):
        print(f'{i + 1}/{len(img_files)} {file_name}')
        json_file = img_full_path / f"{file_name}.json"
        img_file = img_full_path / f"{file_name}.png"
        try:
            frames = json_load(str(json_file))
            img = cv2.imread(str(img_file))
            if img is None:
                print(f"{YELLOW}Warning: Unable to read image {img_file}{END}")
                continue
        except Exception as e:
            print(f"{RED}Error loading {file_name}: {e}{END}")
            continue

        for pos_name, status in frames.items():
            if pos_name not in frame_dict:
                print(f'{pos_name} not in frames')
                continue

            if frame_dict[pos_name]['model_used'] != model_name:
                continue

            print(f'    {model_name} {pos_name} {status}')

            xywh = frame_dict[pos_name]['xywh']

            # Save original cropped image
            img_crop = crop_img(img, xywh, resize=resize)
            log_dir = img_frame_log_path / model_name
            log_dir.mkdir(parents=True, exist_ok=True)
            cv2.imwrite(str(log_dir / f"{status}_{pos_name}_{file_name}.png"), img_crop)

            # Process and save variations
            variant_dir = img_frame_path / model_name / status
            variant_dir.mkdir(parents=True, exist_ok=True)

            for shift_y in config['shift_values']:
                for shift_x in config['shift_values']:
                    shifted_crop = crop_img(img, xywh, shift=(shift_x, shift_y), resize=resize)
                    for brightness in config['brightness_values']:
                        for contrast in config['contrast_values']:
                            img_variant = controller(shifted_crop, brightness, contrast)
                            output_filename = f"{file_name}_{pos_name}_{status}_{shift_y}_{shift_x}_{brightness}_{contrast}.png"
                            cv2.imwrite(str(variant_dir / output_filename), img_variant)


def create_model(config: dict, model_name: str, img_height: int, img_width: int, batch_size: int, epochs: int,
                 paths: dict) -> None:
    # Use pathlib for cross-platform path handling
    data_dir = pathlib.Path(paths['img_frame']) / model_name

    # delete file if config['max_file']>0
    for class_name in os.listdir(data_dir):
        class_dir = data_dir / class_name
        if len(os.listdir(class_dir)) > config['max_file']:
            files = random.sample(os.listdir(class_dir), len(os.listdir(class_dir)) - config['max_file'])
            for file in files:
                os.remove(class_dir / file)

    # Count images in all subdirectories
    image_count = len(list(data_dir.glob('*/*.png')))
    print(f'image_count = {image_count}')

    train_ds, val_ds = keras.utils.image_dataset_from_directory(
        data_dir,
        validation_split=0.2,
        subset="both",
        seed=123,
        image_size=(img_height, img_width),
        batch_size=batch_size
    )

    class_names = train_ds.class_names
    print('class_names =', class_names)

    # Save class names info to JSON
    model_path = pathlib.Path(paths['model'])
    model_path.mkdir(parents=True, exist_ok=True)
    with open(model_path / f"{model_name}.json", 'w') as file:
        json.dump({"class_names": class_names, "img_size": (img_width, img_height)}, file, indent=4)

    # Visualize a sample of the training data
    plt.figure(figsize=(20, 10))
    for images, labels in train_ds.take(1):
        for i in range(min(32, len(images))):
            ax = plt.subplot(4, 8, i + 1)
            plt.imshow(images[i].numpy().astype("uint8"))
            plt.title(class_names[labels[i]])
            plt.axis("off")
    plt.savefig(str(model_path / f"{model_name}.png"))
    plt.close()

    # Use TensorFlow's AUTOTUNE for data prefetching
    AUTOTUNE = tf.data.AUTOTUNE
    train_ds = train_ds.cache().shuffle(1000).prefetch(buffer_size=AUTOTUNE)
    val_ds = val_ds.cache().prefetch(buffer_size=AUTOTUNE)

    # Build the model
    num_classes = len(class_names)
    data_augmentation = keras.Sequential(
        [
            layers.RandomFlip("horizontal",
                              input_shape=(img_height,
                                           img_width,
                                           3)),
            layers.RandomRotation(0.1),
            layers.RandomZoom(0.1),
        ]
    )

    model = Sequential([
        data_augmentation,
        layers.Rescaling(1. / 255),
        layers.Conv2D(16, 3, padding='same', activation='relu'),
        layers.MaxPooling2D(),
        layers.Conv2D(32, 3, padding='same', activation='relu'),
        layers.MaxPooling2D(),
        layers.Conv2D(64, 3, padding='same', activation='relu'),
        layers.MaxPooling2D(),
        layers.Dropout(0.2),
        layers.Flatten(),
        layers.Dense(128, activation='relu'),
        layers.Dense(num_classes, name="outputs")
    ])

    model.compile(optimizer='adam',
                  loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True),
                  metrics=['accuracy'])
    model.summary()

    history = model.fit(train_ds, validation_data=val_ds, epochs=epochs)

    # Visualize training results
    acc = history.history.get('accuracy', [])
    val_acc = history.history.get('val_accuracy', [])
    loss = history.history.get('loss', [])
    val_loss = history.history.get('val_loss', [])
    epochs_range = range(epochs)
    plt.figure(figsize=(8, 8))
    plt.subplot(1, 2, 1)
    plt.plot(epochs_range, acc, label='Training Accuracy')
    plt.plot(epochs_range, val_acc, label='Validation Accuracy')
    plt.legend(loc='lower right')
    plt.title('Training and Validation Accuracy')
    plt.subplot(1, 2, 2)
    plt.plot(epochs_range, loss, label='Training Loss')
    plt.plot(epochs_range, val_loss, label='Validation Loss')
    plt.legend(loc='upper right')
    plt.title('Training and Validation Loss')
    plt.savefig(str(model_path / f"{model_name}.png"))
    plt.close()

    # Save the trained model
    model.save(str(model_path / f"{model_name}.keras"))
    model.save(str(model_path / f"{model_name}.h5"))

    # Clean up the image frame folder after training
    shutil.rmtree(str(paths['img_frame'] / model_name))


def training_(inspection_name: str, config: dict) -> None:
    projects_directory = pathlib.Path(config['projects_directory'])
    # Create a base directory for the inspection
    inspection_dir = projects_directory / f"auto_inspection_data__{inspection_name}"

    # Define all relevant subdirectories in a dictionary
    paths = {
        'img_full': inspection_dir / "img_full",
        'img_frame': inspection_dir / "img_frame",
        'img_frame_log': inspection_dir / "img_frame_log",
        'model': inspection_dir / "model"
    }
    # Ensure directories exist
    for path in paths.values():
        path.mkdir(parents=True, exist_ok=True)

    # List existing trained models (h5 files)
    model_list = [p.stem for p in (paths['model']).glob("*.h5")]
    print(f"\n{CYAN}===========  {inspection_name}  ==========={END}")
    print(f'model.h5 (ที่มี) = {len(model_list)} {model_list}')

    # Load JSON data with frame and model info
    frames_json_path = inspection_dir / 'frames pos.json'
    json_data = json_load(str(frames_json_path))
    frame_dict = json_data['frames']
    model_dict = json_data['models']

    wait_training_path = inspection_dir / 'wait_training.json'
    wait_training_dict = json_load(str(wait_training_path), {})

    for model_name, model in model_dict.items():
        if not wait_training_dict.get(model_name, True):
            print(f'continue {model_name}')
            continue

        print(f"\n{model_name}: {model}")
        t1 = datetime.now()
        print('-------- >>> crop_img <<< ---------')

        save_img(config, model_name, frame_dict, {k: str(v) for k, v in paths.items()})
        t2 = datetime.now()
        print(f'{t2 - t1} เวลาที่ใช้ในการเปลียน img_full เป็น shift_img')
        print('------- >>> training... <<< ---------')
        create_model(config, model_name, config['img_height'], config['img_width'],
                     config['batch_size'], config['epochs'],
                     {k: paths[k] for k in ['img_frame', 'model']})
        json_update(str(wait_training_path), {model_name: False})
        t3 = datetime.now()
        print(f'{t2 - t1} เวลาที่ใช้ในการเปลียน img_full เป็น shift_img')
        print(f'{t3 - t2} เวลาที่ใช้ในการ training')
        print(f'{t3 - t1} เวลาที่ใช้ทั้งหมด\n')


def training(*inspection_names: str, config: dict) -> None:
    for inspection_name in inspection_names:
        training_(inspection_name, config)


if __name__ == '__main__':
    training(
        "QC7-7990-000",
        "QD1-1988-000",
        "POWER-SUPPLY-FIXING-UNIT",
        "POWER-SUPPLY-FIXING-UNIT2",
        config={
            'projects_directory': r'C:\PythonProjects',
            'batch_size': 32,
            'img_height': 180,
            'img_width': 180,
            'epochs': 5,
            'shift_values': [-4, -2, 0, 2, 4],
            'brightness_values': [-24, -12, 0, 12, 24],
            'contrast_values': [-12, -6, 0, 6, 12],
            'max_file': 20000,
        }
    )



import numpy as np
import cv2


def perpendicular_line(xy1, xy2):
    midpoint = (xy1 + xy2) / 2
    delta_y = xy2[1] - xy1[1]
    delta_x = xy2[0] - xy1[0]

    if delta_x == 0:
        slope_perpendicular = 0
        intercept = midpoint[1]
    else:
        # m = dy/dx
        slope = delta_y / delta_x
        slope_perpendicular = -1 / slope

        # y = mx + c
        # c = y - mx
        intercept = midpoint[1] - slope_perpendicular * midpoint[0]

    return slope_perpendicular, intercept


def intersection_point(line1, line2):
    m1, c1 = line1
    m2, c2 = line2

    if m1 == m2:
        raise ValueError("The lines are parallel and do not intersect.")

    x = (c2 - c1) / (m1 - m2)
    y = m1 * x + c1

    return np.array([x, y])


def rotate(image, phi, center):
    if abs(phi) < 0.01:
        print(f"func rotate, phi={phi}, don't rotate, phi<0.01")
    else:
        print(f'func rotate, phi={phi}')
        h, w = image.shape[:2]
        rotate_matrix = cv2.getRotationMatrix2D(center=center.tolist(), angle=phi, scale=1)
        image = cv2.warpAffine(src=image, M=rotate_matrix, dsize=(w, h))
    return image


def fine_mark(image, mark, xywh):
    WH_ = np.array(image.shape[1::-1])
    wh_ = np.array(mark.shape[1::-1])
    XY = xywh[:2]
    WH = xywh[2:]
    XY1_ = ((XY - WH / 2) * WH_).astype(int)
    XY2_ = ((XY + WH / 2) * WH_).astype(int)
    max_score = 0
    xy_ = None

    result = cv2.matchTemplate(image[XY1_[1]:XY2_[1], XY1_[0]:XY2_[0]], mark, cv2.TM_CCOEFF_NORMED)
    loc = np.where(result > 0.7)
    for pt in zip(*loc):
        scrolling_xy_ = pt[::-1]
        if max_score < result[*pt]:
            max_score = result[*pt]
            xy_ = XY1_ + scrolling_xy_ + wh_ / 2

    if type(xy_) != type(None):
        xy = xy_ / WH_
        return xy


def adj_image(image, model_name, mark_dict):
    h, w, _ = image.shape

    mark1 = cv2.imread(f'data/{model_name}/m1.png')
    mark2 = cv2.imread(f'data/{model_name}/m2.png')
    xy1 = mark_dict['m1']['xy']
    xy2 = mark_dict['m2']['xy']
    m1_xy = fine_mark(image, mark1, mark_dict['m1']['xywh_around'])
    m2_xy = fine_mark(image, mark2, mark_dict['m2']['xywh_around'])
    # print('m1_xy =', m1_xy, '  m2_xy =', m2_xy)

    if m1_xy is None or m2_xy is None:
        print('no mark point_be')
        return image
    else:
        # ------ rotate and move ------
        xy1_ = (xy1 * [w, h])
        xy2_ = (xy2 * [w, h])
        m1_xy_ = (m1_xy * [w, h])
        m2_xy_ = (m2_xy * [w, h])
        center_for_rotate = (m1_xy_ + m2_xy_) // 2

        default_phi = np.degrees(np.arctan2((xy2_[1] - xy1_[1]), (xy2_[0] - xy1_[0])))
        phi = np.degrees(np.arctan2((m2_xy_[1] - m1_xy_[1]), (m2_xy_[0] - m1_xy_[0])))

        # print((m1_xy_, xy1_))
        # print(m2_xy_, xy2_)
        line1 = perpendicular_line(m1_xy_, xy1_)
        line2 = perpendicular_line(m2_xy_, xy2_)
        intersection = intersection_point(line1, line2)

        print('default_phi', default_phi)
        print('phi', phi)
        image_ro = rotate(image, (phi - default_phi), center=np.array(intersection))
        # image_ro = rotate(image, (phi - default_phi), center=np.array(center_for_rotate))

        return image_ro

        # ------ move ------
        # m1_xy = fine_mark(image, mark1, mark_dict['m1']['xywh_around'])
        # m2_xy = fine_mark(image, mark2, mark_dict['m2']['xywh_around'])
        # if point_m1_af is None or point_m2_af is None:
        #     print('no mark point_af <<<<<')
        #     return
        #
        # x = (frame.marks['m1'].xpx(h, w) - point_m1_af[0] + frame.marks['m2'].xpx(h, w) - point_m2_af[0]) // 2
        # y = (frame.marks['m1'].ypx(h, w) - point_m1_af[1] + frame.marks['m2'].ypx(h, w) - point_m2_af[1]) // 2
        # image = overlay(image, image_ro, (x, y))
        # return image



import csv
import json
import os.path
import matplotlib.pyplot as plt


def load_frames_data(file_path):
    """Load 'frames pos.json' data."""
    if not os.path.exists(file_path):
        print(f"Error: {file_path} not found.")
        return {}

    try:
        with open(file_path, 'r') as file:
            return json.load(file)
    except json.JSONDecodeError:
        print(f"Error: Failed to parse JSON from {file_path}. Ensure the file structure is correct.")
        return {}



def read_results(file_path):
    """Read data from 'result.txt'."""
    if not os.path.exists(file_path):
        print(f"Error: {file_path} not found.")
        return []

    try:
        with open(file_path, 'r') as f:
            return f.readlines()
    except Exception as e:
        print(f"Error reading file {file_path}: {e}")
        return []



def process_data(frames_data, results):
    """Process results and frames data to extract required metrics."""
    frames = frames_data.get("frames", {})
    frame_names = set()
    table_data = []
    plot_data = {comp: [] for comp in frames.keys()}
    img_names = []

    for line in results:
        try:
            img_name, json_data = line.strip().split("--")
            row_data = {"img_name": img_name}

            # Parse JSON and compute metrics
            for frame_name, vel_data in json.loads(json_data).items():
                frame_names.add(frame_name)
                # Get 'OK' cases for the frame
                ok_case = frames.get(frame_name, {}).get('res_show', {}).get('OK', [])
                ok_case_total_vel = sum(percent for case_name, percent in vel_data.items() if case_name in ok_case)

                row_data[frame_name] = ok_case_total_vel
                plot_data[frame_name].append(ok_case_total_vel)

            img_names.append(img_name)
            table_data.append(row_data)
        except (ValueError, json.JSONDecodeError) as e:
            print(f"Error processing line: {line}. Error: {e}")

    return sorted(frame_names), table_data, plot_data, img_names


def export_to_csv(file_path, frame_names, table_data):
    """Export data to CSV."""
    try:
        with open(file_path, 'w', newline='') as csvfile:
            fieldnames = ["img_name"] + frame_names
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()

            for row in table_data:
                # Ensure missing columns have a blank value
                row = {frame: row.get(frame, "") for frame in fieldnames}
                writer.writerow(row)

        print(f"CSV table successfully saved to {file_path}")
    except Exception as e:
        print(f"Error exporting to CSV: {e}")



def plot_data(plot_data, img_names,output_plot_file):
    """Generate and display plot."""
    plt.figure(figsize=(12, 8))

    for comp, values in plot_data.items():
        plt.scatter(img_names, values, label=comp)  # Scatter plot for readability
        # plt.plot(img_names,values)

    # Customize graph
    plt.title("Status Over Time")
    plt.xlabel("Data")
    plt.ylabel("% OK")
    plt.ylim(0, 100)
    plt.legend(loc="upper right", fontsize=8)
    plt.xticks(rotation=90)
    plt.tight_layout()

    plt.savefig(output_plot_file)
    plt.show()


def summary(main_path):
    # File paths (configurable)
    frames_file = os.path.join(main_path, "frames pos.json")
    results_file = os.path.join(main_path, "img_result", "result.txt")
    output_csv_file = os.path.join(main_path, "img_result.csv")
    output_plot_file = os.path.join(main_path, "img_result_plot.png")

    # Load data
    frames_data = load_frames_data(frames_file)
    results = read_results(results_file)

    # Validate input data
    if not frames_data or not results:
        print("Error: Missing or invalid input files.")
        return

    # Process data and generate outputs
    frame_names, table_data, plot_data_values, img_names = process_data(frames_data, results)
    export_to_csv(output_csv_file, frame_names, table_data)
    plot_data(plot_data_values, img_names,output_plot_file)


if __name__ == "__main__":
    main_path = r'C:\PythonProjects\auto_inspection_data__POWER-SUPPLY-FIXING-UNIT'
    summary(main_path)



theme = {}

# label
theme.update({
    "label": {
        "colours": {
            "normal_text": "#000",
        },
        "font": {
            "name": "Arial",
            "size": "14"
        },
        "misc": {
            "text_horiz_alignment": "left"
        }
    },
    "#model_label": {
        "font": {
            "name": "Arial",
            "size": "18"
        },
    }
})

# button
theme.update({
    "button": {
        "colours": {
            "normal_bg": "#F3F3F3",
            "hovered_bg": "rgb(229,243,255)",
            "disabled_bg": "#F3F3F3",
            "selected_bg": "rgb(204,232,255)",
            "active_bg": "rgb(204,232,255)",
            "normal_text": "#000",
            "hovered_text": "#000",
            "selected_text": "#000",
            "disabled_text": "#A6A6A6",
            "active_text": "#000",
            "normal_border": "#CCCCCC",
            "hovered_border": "#A6A6A6",
            "disabled_border": "#CCCCCC",
            "selected_border": "#A6A6A6",
            "active_border": "#0078D7",
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
            "border_width": "1",
            "shadow_width": "0",
            "tool_tip_delay": "1.0",
            "text_horiz_alignment": "center",
            "text_vert_alignment": "center",
            "text_horiz_alignment_padding": "10",
            "text_vert_alignment_padding": "5",
            "text_shadow_size": "0",
            "text_shadow_offset": "0,0"
        }
    },
    '@panel2_up_button': {
    },
    '#close_button': {
        "colours": {
            "normal_bg": "#0000",
            "hovered_bg": "#ff1212",
            "active_bg": "#e81123",
            "normal_text": "#000",
            "hovered_text": "#FFF",
            "active_text": "#FFF"
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "0,0,0,0",
            "border_width": "0",
            "shadow_width": "0",
        }
    },
    '#minimize_button': {
        "colours": {
            "normal_bg": "#0000",
            "hovered_bg": "#FFFA",
            "active_bg": "#FFFC",
            "normal_text": "#000",
            "hovered_text": "#000",
            "active_text": "#000"
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "0,0,0,0",
            "border_width": "0",
            "shadow_width": "0",
        }
    },
    "#logo_button": {
        "colours": {
            "normal_bg": "#0000",
            "hovered_bg": "#0000",
            "active_bg": "#0000",
            "normal_text": "#0000",
            "hovered_text": "#0000",
            "active_text": "#0000"
        },
        "images": {
            "normal_image": {
                "path": "ui/icon.png",
                "sub_surface_rect": "0,0,30,30"
            },

        },
    },
    "#buttom_bar": {
        "colours": {
            "normal_bg": "#0000",
            "hovered_bg": "#0000",
            "active_bg": "#0000",
            "normal_text": "#000",
            "hovered_text": "#000",
            "active_text": "#000"
        },
        "misc": {
            "shape": "rectangle",
            "border_width": "0",
            "shadow_width": "1"
        }
    }
})

# panel
theme.update({
    "panel": {
        "colours": {
            "dark_bg": "#F9F9F9",
            "normal_border": "#888"
        },
    }
})

# UISelectionList
theme.update({
    "selection_list": {
        "colours": {
            "dark_bg": "#F9F9F9",
            "normal_border": "#999999"

        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
        }
    },
    "selection_list.@selection_list_item": {
        "colours": {
            "dark_bg": "#F9F9F9",
            "normal_border": "#999999"
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
        }
    },
    # UIButton
    "@RightClick.#item_list_item": {
        "misc": {
            "text_horiz_alignment": "left",
            "text_horiz_alignment_padding": "10",
        }
    }
})

# drop_down_menu
theme.update({
    "drop_down_menu": {
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
            "open_button_width": "0"
        }
    },
    "drop_down_menu.button": {
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
        }
    },
    "drop_down_menu.#drop_down_options_list": {
        "colours": {
            "dark_bg": "#F9F9F9",
            "normal_border": "#999999"
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
        }
    },
    "drop_down_menu.#drop_down_options_list.button": {
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
        }
    },
})

# UIVerticalScrollBar
theme.update({
    "vertical_scroll_bar": {
        "colours": {
            "normal_bg": "rgb(205, 205, 205)",
            "hovered_bg": "rgb(166, 166, 166)",
            "selected_bg": "#25292e",
            "active_bg": "rgb(96, 96, 96)",
            "dark_bg": "#f0f0f0",
            "normal_text": "#c5cbd8",
            "hovered_text": "#FFFFFF",
            "selected_text": "#FFFFFF",
            "disabled_text": "#6d736f"
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "10",
            "enable_arrow_buttons": "0"
        }
    },
    "vertical_scroll_bar.#sliding_button": {
        "colours": {
            "normal_bg": "rgb(205, 205, 205)",
            "hovered_bg": "rgb(166, 166, 166)",
            "selected_bg": "#25292e",
            "active_bg": "rgb(96, 96, 96)",
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "10",
            "border_width": "1"
        }
    }
})

# text_box
theme.update({
    "text_box": {
        "colours": {
            "dark_bg": "#ced1ff,#e9efc8,30",
            "selected_bg": "#00F",
            "normal_border": "rgb(130, 170, 255)",
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "10",
            "border_width": "2"
        }
    },
    "passrate_textbox": {
        "font": {
            "name": "Arial",
            "size": "50"
        },
    }

})

# UISelectionList
theme.update({
    "selection_list": {
        "colours": {
            "dark_bg": "#FFF",
            "normal_border": "#999"
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
            "border_width": "1",
            "list_item_height": "25"
        }
    }
    # UIVerticalScrollBar

})

# UIWindow
theme.update({
    "window": {
        "colours": {
            "dark_bg": "#FFFF",
            "normal_border": "#999"
        },

        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
            "title_bar_height": "25"
        }
    },
    "window.#title_bar": {
        "colours": {
            "normal_bg": "#e4ebf499",
            "hovered_bg": "#f4fbf4BB",
            "active_bg": "#e4ebf4",
            "normal_text": "#000",
            "hovered_text": "#000",
            "active_text": "#000"
            #         "normal_bg": "#F3F3F3",
            #         "hovered_bg": "rgb(229,243,255)",
            #         "disabled_bg": "#F3F3F3",
            #         "selected_bg": "rgb(204,232,255)",
            #         "active_bg": "rgb(204,232,255)",
            #         "normal_text": "#000",
            #         "hovered_text": "#000",
            #         "selected_text": "#000",
            #         "disabled_text": "#A6A6A6",
            #         "active_text": "#000",
            #         "normal_border": "#CCCCCC",
            #         "hovered_border": "#A6A6A6",
            #         "disabled_border": "#CCCCCC",
            #         "selected_border": "#A6A6A6",
            #         "active_border": "#0078D7",
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5,0,0,0",

        }
    },
    "window.#close_button": {
        "colours": {
            "normal_bg": "#e4ebf4",
            "hovered_bg": "#ff1212",
            "active_bg": "#e81123",
            "normal_text": "#000",
            "hovered_text": "#FFF",
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "0,5,0,0",
        }
    }
})

# UIFileDialog
theme.update({
    # UITextEntryLine
    "file_dialog.#file_path_text_line": {
        "colours": {
            "dark_bg": "#FF93",
            "selected_bg": "#0078d7",
            "normal_text": "#000",
            "selected_text": "#FFF",
        }},
    # UISelectionList
    "#file_dialog.#file_display_list": {
        "colours": {
            "dark_bg": "#FFF",
            "normal_border": "#999"
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
            "list_item_height": "25"
        }
    },
    # UIButton
    "#file_dialog.#file_display_list.#directory_list_item": {
        "colours": {
            "normal_bg": "#FF93",
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
        }
    },
    # UIButton
    "file_dialog.#file_display_list.#file_list_item": {
        "colours": {
            "normal_bg": "#FFF0",
        },
        "misc": {
            "shape": "rounded_rectangle",
            "shape_corner_radius": "5",
        }
    }
})

if __name__ == '__main__':
    import auto_inspection

    auto_inspection.main()



import json
import os
from datetime import datetime
import re
import pathlib
from pathlib import Path
from pprint import pprint
import numpy as np
import cv2
import pygame as pg
from hexss.box import Box
from hexss.image.func import pygame_surface_to_numpy
from pygame import Rect, Surface, mouse, MOUSEBUTTONDOWN
import pygame_gui
from pygame_gui import UIManager, UI_FILE_DIALOG_PATH_PICKED, UI_BUTTON_PRESSED, UI_DROP_DOWN_MENU_CHANGED, \
    UI_SELECTION_LIST_NEW_SELECTION
from pygame_gui.core import ObjectID
from pygame_gui.elements import UIPanel, UILabel, UIButton, UIDropDownMenu, UISelectionList
from pygame_gui.windows import UIFileDialog
from keras import models
from hexss.constants.terminal_color import *
from hexss import json_load, json_update
from hexss.image import get_image, crop_img, Image
from .theme import theme
from .textbox_surface import TextBoxSurface, gradient_surface
from .pygame_function import putText, UITextBox
from os.path import join
from .summary_graphs import summary
from PIL import Image as PILImage


def pil_to_numpy_strict(img):
    if hasattr(img, "image") and not isinstance(img, PILImage.Image):
        img = img.image

    w, h = img.size
    bands = len(img.getbands())  # 1 for L, 3 for RGB, 4 for RGBA
    buf = np.frombuffer(img.tobytes(), dtype=np.uint8)
    if bands == 1:
        return buf.reshape(h, w)
    else:
        return buf.reshape(h, w, bands)


class RightClick:
    def __init__(self, app, window_size):
        self.app = app
        self.options_list = set()
        self.selection = None
        self.window_size = np.array(window_size)

    def add_options_list(self, new_options_list):
        self.options_list = self.options_list.union(new_options_list)

    def remove_options_list(self, new_options_list):
        self.options_list = self.options_list - new_options_list

    def create_selection(self, mouse_pos, options_list, object_id=None):
        # 1 character ≈ 8px
        max_character = 0
        for option in options_list:
            max_character = len(option) if max_character < len(option) else max_character

        if len(options_list) > 0:
            selection_size = (max_character * 8 + 20, 6 + 25 * len(options_list))
            pos = np.minimum(mouse_pos, self.window_size - selection_size)
            options_list = list(options_list)
            options_list.sort(),
            self.selection = UISelectionList(
                Rect(pos, selection_size),
                item_list=options_list,
                manager=self.app.manager,
                object_id=ObjectID(class_id='@RightClick', object_id=object_id) \
                    if type(object_id) == str else object_id,
            )

    def kill(self):
        if self.selection:
            self.selection.kill()
            self.selection = None

    def events(self, events):
        for event in events:
            if event.type == UI_SELECTION_LIST_NEW_SELECTION and '#RightClick' in event.ui_object_id:
                # print(f"RightClick: {event.text}")
                self.kill()

            if event.type == MOUSEBUTTONDOWN:
                if event.button == 1:  # Left mouse button
                    if self.selection and not self.selection.get_relative_rect().collidepoint(event.pos):
                        self.kill()
                # elif event.button == 3:  # Right mouse button
                #     self.kill()
                #     if self.app.scale_and_offset_button.get_abs_rect().collidepoint(mouse.get_pos()):
                #         self.create_selection(event.pos, {'save config', 'save mark'})
                #     elif self.app.panel1_rect.collidepoint(mouse.get_pos()):
                #         self.create_selection(event.pos, {'zoom to fit'})
                #     else:
                #         self.create_selection(event.pos, self.options_list)


class AutoInspection:
    IMG = np.full((2448, 3264, 3), [150, 150, 150], dtype=np.uint8)

    def update_scaled_img_surface(self):
        self.scaled_img_surface = pg.transform.scale(self.img_surface, (
            int(self.img_surface.get_width() * self.scale_factor),
            int(self.img_surface.get_height() * self.scale_factor)))
        self.img_size_vector = np.array(self.scaled_img_surface.get_size())

    def get_surface_form_np(self, np_img):
        if np_img is None:
            np_img = self.IMG.copy()
        self.img_surface = pg.image.frombuffer(np_img.tobytes(), np_img.shape[1::-1], "BGR")
        self.update_scaled_img_surface()

    def get_surface_form_config_data(self):
        if self.data.get('img') is not None:
            self.np_img = self.data.get('img')
            self.get_surface_form_np(self.np_img)

    def show_rects_to_surface(self, frame_dict, type='frame'):
        if self.is_show_rects:
            for k, v in frame_dict.items() if frame_dict else ():
                xywh = np.array(v.get('xywh'))
                xy = xywh[:2]
                wh = xywh[2:]
                x1y1 = xy - wh / 2
                x1y1_ = x1y1 * self.img_size_vector
                wh_ = wh * self.img_size_vector
                rect = Rect(x1y1_, wh_)

                pg.draw.rect(self.scaled_img_surface, v['color_rect'],
                             rect.inflate(v['width_rect'] * 2 + 1, v['width_rect'] * 2 + 1), v['width_rect'])

                # pg.draw.rect(self.scaled_img_surface, (200, 255, 0), rect, 1)
                pg.draw.line(self.scaled_img_surface, (0, 0, 100), rect.topleft, rect.topright)
                pg.draw.line(self.scaled_img_surface, (0, 0, 100), rect.bottomleft, rect.bottomright)
                pg.draw.line(self.scaled_img_surface, (0, 0, 100), rect.topleft, rect.bottomleft)
                pg.draw.line(self.scaled_img_surface, (0, 0, 100), rect.topright, rect.bottomright)

                if type == 'mark':
                    wh = wh * v['k']
                    x1y1 = xy - wh / 2
                    x1y1_ = x1y1 * self.img_size_vector
                    wh_ = wh * self.img_size_vector
                    rect = Rect(x1y1_, wh_)

                    pg.draw.rect(self.scaled_img_surface, (200, 255, 0), rect, 1)

            for k, v in frame_dict.items() if frame_dict else ():
                xywh = np.array(v.get('xywh'))
                xy = xywh[:2]
                wh = xywh[2:]
                x1y1 = xy - wh / 2
                x1y1_ = x1y1 * self.img_size_vector
                wh_ = wh * self.img_size_vector
                rect = Rect(x1y1_, wh_)

                text = f"{k}"
                if v.get('highest_score_name'):
                    text += f"\n{v['highest_score_name']} {v['highest_score_percent']}"
                putText(self.scaled_img_surface, text, rect.topleft, 16, (0, 0, 255), (255, 255, 255),
                        anchor='bottomleft')

                if type == 'frame':
                    if k in self.debug_class_name.keys():
                        class_name = self.debug_class_name[k]
                        putText(self.scaled_img_surface, f'{class_name}', rect.move((0, 10)).topleft, 16,
                                (220, 0, 190), (255, 255, 255), anchor='bottomleft')

    def model_name_dir(self):
        return join(self.data['config']['projects_directory'], f"auto_inspection_data__{self.data['model_name']}")

    def __init__(self, data, ui=True):
        self.save_result = 0
        self.is_show_rects = True
        self.data = data
        self.config = data['config']
        self.xfunction = self.config.get('xfunction')
        self.resolution = self.config.get('resolution')
        if self.resolution == '1920x1080':
            self.window_size = np.array([1920, 1080])
        else:
            self.window_size = np.array([800, 480])
        self.data['model_name'] = self.config['model_name']

        self.np_img = self.IMG.copy()
        self.mark_template_im = None
        # if ui:
        pg.init()
        pg.display.set_caption('Auto Inspection')
        self.clock = pg.time.Clock()

        self.display = pg.display.set_mode(self.window_size.tolist(), pg.FULLSCREEN if self.config['fullscreen'] else 0)
        self.manager = UIManager(self.display.get_size(), theme_path=theme)
        self.right_click = RightClick(self, self.window_size.tolist())
        self.setup_ui()
        self.change_model()

        self.file_name = None
        self.debug_class_name = {}  # key is pos_name, vel is class_name

        self.pass_n = 0
        self.fail_n = 0

        self.buff = '----------'

    def set_name_for_debug(self, file_name=None):
        self.file_name = datetime.now().strftime("%y%m%d %H%M%S") if file_name is None else file_name
        json_path = join(self.model_name_dir(), 'img_full', f'{self.file_name}.json')
        self.debug_class_name = json_load(json_path, {})

    def reset_frame(self):
        for name, frame in self.frame_dict.items() if self.frame_dict else ():
            frame['color_rect'] = (255, 220, 0)
            frame['width_rect'] = 2
            frame['highest_score_name'] = ''
            frame['highest_score_percent'] = ''
            if not frame.get('frame_name'):
                frame['frame_name'] = name
        self.res_textbox.update_text('res', text='-', color=(0, 0, 0))
        self.config['res'] = '-'
        self.setup_NG_details()

    def setup_NG_details(self):
        size_font = 25 if self.resolution == '1920x1080' else 13
        formatted_text = ""
        for k, v in self.frame_dict.items() if self.frame_dict else ():
            if v.get('highest_score_name') in ['', 'OK']:
                continue
            text = f"{k} {v['highest_score_name']}"
            formatted_text += f"<font color='#FF0000' size={size_font}>{text}</font><br>"
        self.res_NG_text_box.set_text(formatted_text)

    def update_status(self):
        self.setup_NG_details()
        self.passrate_textbox.update_text('Pass', text=f': {self.pass_n}')
        self.passrate_textbox.update_text('Fail', text=f': {self.fail_n}')
        self.passrate_textbox.update_text('Pass rate', text=
        f': {self.pass_n / (self.pass_n + self.fail_n) * 100:.2f}%' if self.pass_n or self.fail_n else ': -%')

    def change_model(self):
        self.adj_button.disable()
        self.predict_button.disable()
        self.open_image_button.disable()
        self.save_image_button.disable()
        self.frame_dict = None
        self.model_dict = None
        self.mark_dict = None
        if self.data['model_name'] == '-':
            pass
        else:
            frames_pos_file_path = join(self.model_name_dir(), 'frames pos.json')
            json_data = json_load(frames_pos_file_path, {})

            self.frame_dict: dict = json_data.get('frames')
            self.model_dict: dict = json_data.get('models')
            self.mark_dict: dict = json_data.get('marks')
            self.reset_frame()

            mark_template_path = Path(self.model_name_dir()) / 'mark_template.png'
            if mark_template_path.exists():
                self.mark_template_im = Image(mark_template_path)

            for name, frame in self.mark_dict.items() if self.mark_dict else ():
                frame['color_rect'] = (200, 20, 100)
                frame['width_rect'] = 1
                frame['xy'] = np.array(frame['xywh'][:2])
                frame['wh'] = np.array(frame['xywh'][2:])
                frame['xywh_around'] = np.concatenate((frame['xy'], frame['wh'] * frame['k']))
            for name, model in self.model_dict.items() if self.model_dict else ():
                try:
                    model['model'] = models.load_model(join(self.model_name_dir(), f'model/{name}.h5'))
                except Exception as e:
                    print(f'{YELLOW}Error load model.h5.\n'
                          f'file error <data>/{self.data["model_name"]}/model/{name}.h5{END}')
                    print(PINK, e, END, sep='')
                try:
                    json_file_with_model = json_load(join(self.model_name_dir(), f'model/{name}.json'))
                    if json_file_with_model.get('class_names'):
                        json_file_with_model['model_class_names'] = json_file_with_model.pop('class_names')
                    model.update(json_file_with_model)
                    pprint(model)
                    if model['model_class_names'] != model['class_names']:
                        print(f'{YELLOW}class_names       = {model["class_names"]}')
                        print(f'model_class_names = {model["model_class_names"]}{END}')
                except Exception as e:
                    print(f'{YELLOW}function "load_model" error.\n'
                          f'file error <data>/{self.data["model_name"]}/model/{name}.json{END}')
                    print(PINK, e, END, sep='')

            config = json_load(join(self.model_name_dir(), 'model_config.json'))
            if config.get(self.resolution):
                for k, v in config[self.resolution].items():
                    if k == 'scale_factor':
                        self.scale_factor = v
                    elif k == 'img_offset':
                        self.img_offset = np.array(v)
            self.pass_n = self.fail_n = 0
            self.update_status()

            self.open_image_button.enable()
            self.save_image_button.enable()
            self.adj_button.enable() if self.mark_dict else self.adj_button.disable()
            self.predict_button.enable() if self.model_dict else self.predict_button.disable()

        self.res_textbox.update_text('res', text='-')
        self.config['res'] = '-'
        self.setup_NG_details()

        if self.model_data_dropdown:
            self.model_data_dropdown.kill()
        self.create_model_data_dropdown(self.data['model_name'])

    def predict(self):
        self.res_textbox.update_text('res', text='Wait', color=(255, 255, 0))
        self.config['res'] = 'Wait'
        self.manager.draw_ui(self.display)
        pg.display.update()

        res_surface_text = 'OK'
        wh_ = np.array(self.np_img.shape[1::-1])
        print(self.model_dict)
        for name, frame in self.frame_dict.items() if self.frame_dict else ():
            if self.model_dict[frame['model_used']].get('model'):  # มีไฟล์ model.h5
                model = self.model_dict[frame['model_used']]['model']
                model_class_names = self.model_dict[frame['model_used']]['model_class_names']
                xywh = frame['xywh']
                img_crop = crop_img(self.np_img, xywh, resize=(180, 180))
                img_crop = cv2.cvtColor(img_crop, cv2.COLOR_BGR2RGB)
                img_array = img_crop[np.newaxis, :]
                predictions = model.predict_on_batch(img_array)

                predictions_score_list = predictions[0]  # [ -6.520611   8.118368 -21.86103   22.21528 ]
                exp_x = [1.2 ** x for x in predictions_score_list]
                percent_score_list = [round(x * 100 / sum(exp_x)) for x in exp_x]
                highest_score_number = np.argmax(predictions_score_list)  # 3

                highest_score_name = model_class_names[highest_score_number]
                highest_score_percent = percent_score_list[highest_score_number]

                for k, v in frame['res_show'].items():
                    if highest_score_name in v:
                        highest_score_name = k

                frame['highest_score_name'] = highest_score_name
                frame['highest_score_percent'] = highest_score_percent
                frame['class_names_percent'] = dict(zip(model_class_names, percent_score_list))

                if highest_score_name == 'OK':
                    frame['color_rect'] = (0, 255, 0)
                    frame['width_rect'] = 2
                else:
                    frame['color_rect'] = (255, 0, 0)
                    frame['width_rect'] = 3
                    res_surface_text = 'NG'

        if res_surface_text == 'OK':
            self.pass_n += 1
            self.res_textbox.update_text('res', text='OK', color=(0, 255, 0))
            self.config['res'] = 'OK'
        else:
            self.fail_n += 1
            self.res_textbox.update_text('res', text='NG', color=(255, 0, 0))
            self.config['res'] = 'NG'
        self.update_status()
        self.save_result = 1

    def create_model_data_dropdown(self, start_option='-'):
        is_full_hd = self.resolution == '1920x1080'
        model_data = self.data['model_names'] + ['-']
        rect = Rect(10, 5, 300, 30) if is_full_hd else Rect(10, 0, 200, 30)
        self.model_data_dropdown = UIDropDownMenu(
            model_data, start_option, rect, self.manager,
            anchors={'left_target': self.model_label}
        )

    def panel0_setup(self):
        is_full_hd = self.resolution == '1920x1080'
        # top left
        rect = Rect(5, 5, 52, 30) if is_full_hd else Rect(10, 0, 52, 30)
        self.model_label = UILabel(
            rect, f'Model:', self.manager,
            object_id=ObjectID(class_id='@model_label', object_id='#model_label')
        )
        self.create_model_data_dropdown()
        self.open_image_button = UIButton(
            Rect(10, 5, 70, 30) if is_full_hd else Rect(10, 0, 60, 30),
            'Open...', self.manager,
            anchors={'left_target': self.model_data_dropdown}
        )
        self.save_image_button = UIButton(
            Rect(10, 5, 70, 30) if is_full_hd else Rect(10, 0, 60, 30),
            'Save...', self.manager,
            anchors={'left_target': self.open_image_button}
        )
        self.summary_button = UIButton(
            Rect(10, 5, 70, 30) if is_full_hd else Rect(10, 0, 60, 30),
            'Summary', self.manager,
            anchors={'left_target': self.save_image_button}
        )
        self.open_image_button.disable()
        self.save_image_button.disable()

        # top right
        anchors = {'top': 'top', 'left': 'right', 'bottom': 'top', 'right': 'right'}
        rect = Rect(-50, 0, 50, 40) if is_full_hd else Rect(-40, 0, 40, 30)
        self.close_button = UIButton(
            rect, f'X', self.manager,
            object_id=ObjectID(class_id='@close_button', object_id='#close_button'),
            anchors=anchors
        )
        self.minimize_button = UIButton(
            rect, f'—', self.manager,
            object_id=ObjectID(class_id='@minimize_button', object_id='#minimize_button'),
            anchors=anchors | {'right_target': self.close_button}
        )

        # bottom left
        anchors = {'top': 'bottom', 'left': 'left', 'bottom': 'bottom', 'right': 'left'}
        self.fps_button = UIButton(
            Rect(0, -30, 80, 30) if is_full_hd else Rect(0, -20, 80, 20),
            '', self.manager,
            object_id=ObjectID(class_id='@fps_button', object_id='#buttom_bar'),
            anchors=anchors
        )
        self.mouse_pos_button = UIButton(
            Rect(0, -30, 110, 30) if is_full_hd else Rect(0, -20, 110, 20),
            '', self.manager,
            object_id=ObjectID(class_id='@mouse_pos_button', object_id='#buttom_bar'),
            anchors=anchors | {'left_target': self.fps_button}
        )
        self.scale_and_offset_button = UIButton(
            Rect(0, -30, 120, 30) if is_full_hd else Rect(0, -20, 120, 20),
            '', self.manager,
            object_id=ObjectID(class_id='@scale_and_offset_button', object_id='#buttom_bar'),
            anchors=anchors | {'left_target': self.mouse_pos_button}
        )
        self.file_name_button = UIButton(
            Rect(0, -30, 130, 30) if is_full_hd else Rect(0, -20, 130, 20),
            '', self.manager,
            object_id=ObjectID(class_id='@file_name_button', object_id='#buttom_bar'),
            anchors=anchors | {'left_target': self.scale_and_offset_button}
        )

        # bottom left
        anchors = {'top': 'bottom', 'left': 'right', 'bottom': 'bottom', 'right': 'right'}
        init_txt = pathlib.Path(__file__).with_name('__init__.py').read_text(encoding='utf-8')
        version = re.search(r"__version__\s*=\s*['\"]([^'\"]+)['\"]", init_txt).group(1)
        test = f'Auto Inspection {version}' + (f" x {self.xfunction}" if self.xfunction else '')
        w = (150 / 22 * len(test)) + 10
        self.autoinspection_button = UIButton(
            Rect(-w, -30, w, 30) if is_full_hd else Rect(-w, -20, w, 20),
            test, self.manager,
            object_id=ObjectID(class_id='@auto_inspection_button', object_id='#buttom_bar'),
            anchors=anchors
        )

    def panel0_update(self, events):
        is_full_hd = self.resolution == '1920x1080'
        if is_full_hd:
            self.display.blit(gradient_surface(Rect(0, 0, 720, 40), (150, 200, 255), (255, 250, 230)), (0, 0))
            self.display.blit(gradient_surface(Rect(0, 0, 1200, 40), (255, 250, 230), (211, 229, 250)), (720, 0))

        for event in events:
            if event.type == UI_BUTTON_PRESSED:
                if event.type == pg.QUIT:
                    self.data['play'] = False
                if event.ui_element == self.close_button:
                    self.data['play'] = False
                if event.ui_element == self.minimize_button:
                    pg.display.iconify()
                if event.ui_element == self.open_image_button:
                    self.file_dialog = UIFileDialog(
                        Rect(430, 50, 440, 500) if is_full_hd else Rect(200, 50, 400, 400),
                        self.manager, 'Load Image...', {".png", ".jpg"},
                        join(self.model_name_dir(), 'img_full'),
                        allow_picking_directories=True,
                        allow_existing_files_only=True,
                        object_id=ObjectID(class_id='@file_dialog', object_id='#open_img_full'),
                    )
                if event.ui_element == self.save_image_button:
                    self.set_name_for_debug()
                if event.ui_element == self.summary_button:
                    summary(self.model_name_dir())

            if event.type == pygame_gui.UI_BUTTON_START_PRESS:
                if event.ui_object_id == 'drop_down_menu.#selected_option':
                    model_data = self.data['model_names'] + ['-']
                    self.model_data_dropdown.options_list = []
                    for option in model_data:
                        self.model_data_dropdown.options_list.append((option, option))
                    self.model_data_dropdown.menu_states[
                        'expanded'].options_list = self.model_data_dropdown.options_list
            if event.type == UI_DROP_DOWN_MENU_CHANGED:
                self.data['model_name'] = self.model_data_dropdown.selected_option[0]
                self.change_model()

            if event.type == MOUSEBUTTONDOWN:
                if event.button == 3:  # Right mouse button
                    self.right_click.kill()
                    if self.scale_and_offset_button.get_abs_rect().collidepoint(mouse.get_pos()):
                        self.right_click.create_selection(
                            event.pos, {'save config', 'save mark'},
                            '#RightClick.bottom_bar'
                        )
                    if self.panel1_rect.collidepoint(mouse.get_pos()):
                        options_list = set()
                        options_list = options_list.union({'zoom to fit'})
                        if self.data['model_name'] != '-':
                            panel_mouse_xy_ = np.array(event.pos) - self.panel1_rect.topleft
                            img_wh_ = np.array(self.np_img.shape[1::-1])
                            panel_wh_ = np.array(self.panel1_rect.size)
                            img_scale_wh_ = img_wh_ * self.scale_factor

                            # Equation in "Equation/fine img_mouse_xy_.png"
                            img_mouse_xy_ = img_scale_wh_ / 2 - self.img_offset - panel_wh_ / 2 + panel_mouse_xy_
                            img_mouse_xy = img_mouse_xy_ / img_scale_wh_

                            for name, frame in self.frame_dict.items() if self.frame_dict else ():
                                xywh = np.array(frame['xywh'])
                                xy = xywh[:2]
                                wh = xywh[2:]
                                if np.all((xy - wh / 2 <= img_mouse_xy) & (img_mouse_xy <= xy + wh / 2)):
                                    class_names = self.model_dict[frame['model_used']]['class_names']
                                    options_list = options_list.union(
                                        {f"add data {name}->{class_name}" for class_name in class_names}
                                    )

                        self.right_click.create_selection(
                            event.pos,
                            options_list,
                            '#RightClick.on_panel_1'
                        )

            # right click and select click
            if event.type == UI_SELECTION_LIST_NEW_SELECTION:
                if event.ui_object_id == '#RightClick.bottom_bar':
                    if self.data['model_name'] != '-':
                        if event.text == 'save config':
                            json_update(join(self.model_name_dir(), 'model_config.json'), {
                                f"{self.resolution}": {
                                    "scale_factor": self.scale_factor,
                                    "img_offset": self.img_offset.tolist()
                                }
                            })
                        if event.text == 'save mark':
                            for name, mark in self.mark_dict.items() if self.mark_dict else ():
                                xywh = mark['xywh']
                                img = crop_img(self.np_img, xywh, resize=(180, 180))
                                cv2.imwrite(join(self.model_name_dir(), f'{name}.png'), img)

                if event.ui_object_id == '#RightClick.on_panel_1':
                    if event.text == 'zoom to fit':
                        data = json_load(join(self.model_name_dir(), 'model_config.json'), {
                            f"{self.resolution}": {
                                "scale_factor": 1,
                                "img_offset": [0, 0]
                            }
                        })
                        self.scale_factor = data[self.resolution]['scale_factor']
                        self.img_offset = np.array(data[self.resolution]['img_offset'])

                    if 'add data ' in event.text:
                        pos_name, class_name = event.text.split('add data ')[1].split('->')
                        if self.file_name:
                            cv2.imwrite(join(self.model_name_dir(), f'img_full/{self.file_name}.png'), self.np_img)
                            self.debug_class_name = json_update(
                                join(self.model_name_dir(), f'img_full/{self.file_name}.json'),
                                {pos_name: class_name}
                            )
                            json_update(
                                join(self.model_name_dir(), 'wait_training.json'),
                                {self.frame_dict[pos_name]['model_used']: True}
                            )

        t = f'{pg.mouse.get_pos()}'
        t += ' 1' if self.panel1_rect.collidepoint(pg.mouse.get_pos()) else ''
        t += ' 2' if self.panel2_rect.collidepoint(pg.mouse.get_pos()) else ''
        self.fps_button.set_text(f'fps: {round(self.clock.get_fps())}')
        self.mouse_pos_button.set_text(t)
        self.scale_and_offset_button.set_text(f'{round(self.scale_factor, 2)} {self.img_offset.astype(int)}')
        self.file_name_button.set_text(f'{self.file_name}')

    def panel1_setup(self):
        is_full_hd = self.resolution == '1920x1080'
        self.panel1_rect = Rect(0, 40, 1347, 1010) if is_full_hd else Rect(0, 30, 600, 430)
        self.panel1_surface = Surface(self.panel1_rect.size)

        self.scale_factor = 1
        self.img_offset = np.array([0, 0])

        self.moving = False

    def panel1_update(self, events):
        self.panel1_surface.fill((200, 200, 200))
        if self.panel1_rect.collidepoint(mouse.get_pos()):
            self.canter_img_pos = np.array(self.panel1_rect.size) / 2 + self.img_offset
            self.left_img_pos = self.canter_img_pos - self.img_size_vector / 2
        for event in events:
            if event.type == pg.MOUSEBUTTONDOWN:
                if event.button == 2:
                    if self.panel1_rect.collidepoint(mouse.get_pos()):
                        self.moving = True

            if event.type == pg.MOUSEWHEEL:
                if self.panel1_rect.collidepoint(mouse.get_pos()):
                    factor = (1 + event.y / 10)

                    self.scale_factor *= factor
                    a = self.img_size_vector
                    a1 = mouse.get_pos() - self.left_img_pos
                    b = a * factor
                    b1 = a1 * factor
                    canter_img_pos_b = mouse.get_pos() - b1 + b / 2
                    offset = self.canter_img_pos - canter_img_pos_b
                    self.img_offset = self.img_offset - offset

                    # update img_size_vector
                    self.scaled_img_surface = pg.transform.scale(self.img_surface, (
                        int(self.img_surface.get_width() * self.scale_factor),
                        int(self.img_surface.get_height() * self.scale_factor)
                    ))
                    self.img_size_vector = np.array(self.scaled_img_surface.get_size())

            # moving
            if self.moving:
                if event.type == pg.MOUSEMOTION:  # move mouse
                    if event.buttons[1]:
                        self.img_offset += np.array(event.rel)
                if event.type == pg.MOUSEBUTTONUP and event.button == 2:
                    self.moving = False
        if self.mark_dict is not None:
            self.show_rects_to_surface(self.mark_dict, 'mark')
        self.show_rects_to_surface(self.frame_dict, 'frame')
        self.panel1_surface.blit(self.scaled_img_surface,
                                 ((self.panel1_rect.size - self.img_size_vector) / 2 + self.img_offset).tolist())

    def panel2_setup(self):
        is_full_hd = self.resolution == '1920x1080'
        self.panel2_up_rect = Rect(1347, 40, 573, 90) if is_full_hd else Rect(600, 30, 200, 72)
        self.panel2_up = UIPanel(self.panel2_up_rect, manager=self.manager)

        self.panel2_rect = Rect(1347, 127, 573, 925) if is_full_hd else Rect(600, 99, 200, 363)
        self.panel2 = UIPanel(self.panel2_rect, manager=self.manager)

        if is_full_hd:
            self.capture_button = UIButton(Rect(10, 10, 100, 50), 'Capture', container=self.panel2_up, )
            self.auto_cap_button = UIButton(Rect(10, 0, 100, 20), 'Auto', container=self.panel2_up, anchors={
                'top_target': self.capture_button
            })
            self.load_button = UIButton(Rect(10, 10, 100, 70), 'Load Image', container=self.panel2_up, anchors={
                'left_target': self.capture_button
            })
            self.adj_button = UIButton(Rect(10, 10, 100, 70), 'Adj Image', container=self.panel2_up, anchors={
                'left_target': self.load_button
            })
            self.predict_button = UIButton(Rect(10, 10, 100, 70), 'Predict', container=self.panel2_up, anchors={
                'left_target': self.adj_button
            })
            self.capture_predict_button = UIButton(Rect(10, 10, 100, 70), 'Cap&Predict', container=self.panel2_up,
                                                   anchors={
                                                       'left_target': self.predict_button
                                                   })
        else:
            self.capture_button = UIButton(Rect(6, 6, 60, 45), 'Capture', container=self.panel2_up, )
            self.auto_cap_button = UIButton(Rect(6, 0, 60, 15), 'Auto', container=self.panel2_up, anchors={
                'top_target': self.capture_button
            })
            self.load_button = UIButton(Rect(0, 6, 49, 30), 'Load', container=self.panel2_up, anchors={
                'left_target': self.capture_button
            })
            self.adj_button = UIButton(Rect(0, 36, 49, 30), 'Adj', container=self.panel2_up, anchors={
                'left_target': self.capture_button
            })
            self.predict_button = UIButton(Rect(0, 6, 79, 30), 'Predict', container=self.panel2_up, anchors={
                'left_target': self.load_button,
            })
            self.capture_predict_button = UIButton(Rect(0, 35, 79, 30), 'Cap&Pre_', container=self.panel2_up, anchors={
                'left_target': self.load_button,
            })
        self.file_dialog = None
        self.adj_button.disable()
        self.predict_button.disable()

        self.res_textbox = TextBoxSurface(
            Rect((self.panel2_rect.w - 300) / 2, 12, 300, 150) if is_full_hd \
                else Rect((self.panel2_rect.w - 190) / 2, 5, 190, 80),
            container=self.panel2,
        )
        self.res_textbox.add_text(
            'res', text='-', color=(0, 0, 0),
            font_name='Arial', font_size=130 if is_full_hd else 50
        )

        self.passrate_textbox = TextBoxSurface(
            Rect((self.panel2_rect.w - 550) / 2, 170, 550, 180) if is_full_hd \
                else Rect((self.panel2_rect.w - 190) / 2, 84, 190, 90),
            container=self.panel2
        )

        self.passrate_textbox.add_text(
            'Pass_', 'Pass',
            (50, 10) if is_full_hd else (10, 5),
            (0, 200, 0),
            font_name='Arial', font_size=40 if is_full_hd else 20,
            anchor='topleft'
        )
        self.passrate_textbox.add_text(
            'Pass', text=': 0',
            xy=(300, 10) if is_full_hd else (100, 5),
            color=(0, 200, 0),
            font_name='Arial', font_size=40 if is_full_hd else 20,
            anchor='topleft'
        )
        self.passrate_textbox.add_text(
            'Fail_', text='Fail',
            xy=(50, 60) if self.resolution == '1920x1080' else (10, 30),
            color=(255, 0, 0),
            font_name='Arial', font_size=40 if is_full_hd else 20,
            anchor='topleft'
        )
        self.passrate_textbox.add_text(
            'Fail', text=': 0',
            xy=(300, 60) if is_full_hd else (100, 30),
            color=(255, 0, 0),
            font_name='Arial', font_size=40 if is_full_hd else 20,
            anchor='topleft'
        )
        self.passrate_textbox.add_text(
            'Pass rate_', text='Pass rate',
            xy=(50, 110) if is_full_hd else (10, 55),
            color=(0, 0, 0),
            font_name='Arial', font_size=40 if is_full_hd else 20,
            anchor='topleft'
        )
        self.passrate_textbox.add_text(
            'Pass rate', text=': -%',
            xy=(300, 110) if is_full_hd else (100, 55),
            color=(0, 0, 0),
            font_name='Arial', font_size=40 if is_full_hd else 20,
            anchor='topleft'
        )

        # Create a UITextBox inside the panel
        self.res_NG_text_box = UITextBox(
            html_text="",
            relative_rect=Rect(((self.panel2_rect.w - 550) / 2, 357), (550, 555)) if is_full_hd \
                else Rect((self.panel2_rect.w - 190) / 2, 173, 190, 186),
            container=self.panel2
        )

    def panel2_update(self, events):

        def capture_button():
            self.get_surface_form_config_data()
            self.reset_frame()
            self.set_name_for_debug()

        def auto_cap_button():
            self.auto_cap_button.set_text('Auto' if self.auto_cap_button.text == 'Stop' else 'Stop')

        def adj_button():
            if self.mark_template_im is None:
                return

            im2 = Image(self.np_img.copy())

            for mark in self.mark_dict.values():
                box = Box(xywhn=mark['xywh'])
                box.set_size(self.mark_template_im.size)
                mark['im'] = self.mark_template_im.crop(box)
                scale_xywhn = box.scale(mark.get('k', 3)).xywhn
                mark['scale_xywhn'] = scale_xywhn

            pts_src = []
            pts_dst = []
            for mark in self.mark_dict.values():
                xywhn_mark_area = mark['scale_xywhn']
                mark_im = mark['im']
                xy, score = im2.best_match_location(mark_im, xywhn=xywhn_mark_area)
                pts_dst.append(Box(size=im2.size, xywhn=mark['xywh']).xy)
                pts_src.append(xy)

            im2.align_image(pts_src, pts_dst)
            arr = pil_to_numpy_strict(im2.image)
            self.np_img = cv2.cvtColor(arr, cv2.COLOR_RGB2BGR)

            self.reset_frame()

        is_full_hd = self.resolution == '1920x1080'
        data_events = self.data.get('events') or []
        for event in data_events:
            self.data['events'].remove(event)
            if event == 'Capture':
                capture_button()
            if event == 'Auto':
                auto_cap_button()
            if event == 'Adj':
                adj_button()
            if event == 'Predict':
                self.predict()
            if event == 'Capture&Predict':
                capture_button()
                adj_button()
                self.predict()
            if event == 'change_model':
                self.change_model()

        for event in events:
            if event.type == UI_BUTTON_PRESSED:
                if event.ui_element == self.capture_button:
                    capture_button()
                if event.ui_element == self.auto_cap_button:
                    auto_cap_button()
                if event.ui_element == self.load_button:
                    self.auto_cap_button.set_text('Auto')

                    self.file_dialog = UIFileDialog(
                        Rect(1360, 130, 440, 500) if is_full_hd else Rect(200, 50, 400, 400),
                        self.manager, 'Load Image...', {".png", ".jpg"},
                        self.data['config']['projects_directory']
                        if self.data['model_name'] == '-' else self.model_name_dir(),
                        allow_picking_directories=True,
                        allow_existing_files_only=True,
                        object_id=ObjectID(class_id='@file_dialog', object_id='#open_img_other'),
                    )
                if event.ui_element == self.adj_button:
                    adj_button()
                if event.ui_element == self.predict_button:
                    self.predict()
                if event.ui_element == self.capture_predict_button:
                    capture_button()
                    adj_button()
                    self.predict()
            if event.type == UI_FILE_DIALOG_PATH_PICKED:
                # from Load Image
                if event.ui_object_id == '#open_img_other':
                    if '.png' in event.text:
                        print(event.text)
                        self.np_img = cv2.imread(event.text)
                        self.reset_frame()
                        self.set_name_for_debug()
                # from Open Image
                if event.ui_object_id == '#open_img_full':
                    if '.png' in event.text:
                        print('from open_data', event.text)
                        self.np_img = cv2.imread(event.text)
                        self.reset_frame()
                        self.set_name_for_debug(os.path.split(event.text)[1].replace('.png', ''))

            # auto show image
            # if event.type == UI_SELECTION_LIST_NEW_SELECTION and event.ui_object_id == '#file_dialog.#file_display_list':
            #     path = ...
            #     if '.png' in event.text:
            #         self.np_img = cv2.imread(event.text)
            #         self.reset_frame()
            #         self.set_name_for_debug()

        if self.auto_cap_button.text == 'Stop':
            self.get_surface_form_config_data()

    def setup_ui(self, ):
        self.panel0_setup()
        self.panel1_setup()
        self.panel2_setup()

    def handle_events(self):
        events = pg.event.get()
        self.panel0_update(events)
        self.panel1_update(events)
        self.panel2_update(events)
        for event in events:
            self.manager.process_events(event)
            # if event.type != 1024:
            #     print(event)
            if event.type == pygame_gui.UI_BUTTON_ON_HOVERED:
                self.manager.set_active_cursor(pg.SYSTEM_CURSOR_HAND)
            if event.type == pygame_gui.UI_BUTTON_ON_UNHOVERED:
                self.manager.set_active_cursor(pg.SYSTEM_CURSOR_ARROW)

            if event.type == 768:
                if event.dict.get('unicode') and event.unicode in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-<>':
                    self.buff += event.unicode
                    self.buff = self.buff[-20:]

                    # <FM-R511-000>
                    if self.buff[-1] == '>':
                        model_name = self.buff[:-1]
                        model_name = model_name.split('<')[-1]
                        print(model_name)
                        self.buff += '-'

                        self.data['model_name'] = model_name
                        self.change_model()

        self.right_click.events(events)

    def run(self):
        while self.data['play']:
            time_delta = self.clock.tick(60) / 1000.0
            self.display.fill((220, 220, 220))
            self.get_surface_form_np(self.np_img)

            self.handle_events()

            self.display.blit(self.panel1_surface, self.panel1_rect.topleft)

            self.manager.update(time_delta)
            self.manager.draw_ui(self.display)

            pg.display.update()

            if self.save_result:
                self.save_result += 1
                if self.save_result == 4:
                    self.save_result = 0

                    result_path = join(self.model_name_dir(), 'img_result')
                    os.makedirs(result_path, exist_ok=True)
                    cv2.imwrite(join(result_path, f'{self.file_name}.png'), self.np_img)
                    cv2.imwrite(join(result_path, f'{self.file_name}.jpg'), pygame_surface_to_numpy(self.display))

                    result = {}
                    for name, frame in self.frame_dict.items() if self.frame_dict else ():
                        result[name] = frame['class_names_percent']
                    print(result)

                    with open(join(result_path, f'result.txt'), 'a') as f:
                        f.write(f'{self.file_name}--{json.dumps(result)}\n')


def main(data):
    app = AutoInspection(data)
    app.run()



import os
import logging
import time
from hexss import check_packages

check_packages(
    'numpy', 'opencv-python', 'Flask', 'requests', 'pygame-gui',
    'tensorflow', 'keras', 'pyzbar',
    auto_install=True, verbose=False
)

from hexss import json_load, json_update, close_port, get_hostname
from hexss.network import get_all_ipv4
from flask import Flask, render_template, request, jsonify

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

app = Flask(__name__)


@app.route('/api/get_model', methods=['GET'])
def api_get_model():
    data = app.config['data']
    return jsonify({
        "model_names": data.get("model_names", []),
        "model_name": data.get("model_name")
    })


@app.route('/api/change_model', methods=['GET'])
def api_change_model():
    data = app.config['data']
    model_name = request.args.get('model_name', type=str)
    if not model_name:
        return jsonify({"status": "error", "message": "missing model_name"}), 400

    allowed = set(data.get("model_names", []))
    if model_name not in allowed:
        return jsonify({"status": "error", "message": f"invalid model_name: {model_name}"}), 400

    data['model_name'] = model_name
    data['events'].append('change_model')

    print("Change model", model_name)

    return jsonify({"status": "ok", "model_name": model_name}), 200


@app.route('/change_model', methods=['GET'])
def change_model():
    return render_template('change_model.html')


@app.route('/status_robot', methods=['GET'])
def status_robot():
    data = app.config['data']
    return data['robot capture']


@app.route('/', methods=['GET', 'POST'])
def index():
    if request.method == 'POST':
        button_name = request.form.get('button')
        if button_name:
            data = app.config['data']
            data['events'].append(button_name)
            logger.info(f"Button clicked: {button_name}")
    return render_template('index.html')


@app.route('/data', methods=['GET'])
def data():
    def filter_list(d: list):
        filtered = []
        for v in d:
            if isinstance(v, (int, float, bool, str)):
                filtered.append(v)
            if isinstance(v, (list, tuple)):
                filtered.append(filter_list(v))
            if isinstance(v, dict):
                filtered.append(filter_dict(v))
        return filtered

    def filter_dict(d):
        filtered = {}
        for k, v in d.items():
            if isinstance(v, (int, float, bool, str)):
                filtered[k] = v
            if isinstance(v, (list, tuple)):
                filtered[k] = filter_list(v)
            if isinstance(v, dict):
                filtered[k] = filter_dict(v)
        return filtered

    data: dict = app.config['data']
    filtered_data = filter_dict(data)

    return jsonify(filtered_data), 200


def run_server(data):
    log = logging.getLogger('werkzeug')
    log.setLevel(logging.ERROR)

    app.config['data'] = data

    ipv4 = data['config']['ipv4']
    port = data['config']['port']

    if ipv4 == '0.0.0.0':
        for ipv4_ in {'127.0.0.1', *get_all_ipv4(), get_hostname()}:
            logging.info(f"Running on http://{ipv4_}:{port}")
    else:
        logging.info(f"Running on http://{ipv4}:{port}")

    app.run(host=ipv4, port=port, debug=False, use_reloader=False)



from setuptools import setup, find_packages
from AutoInspection import __version__ as version

setup(
    name='AutoInspection',
    version=version,
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    url='https://github.com/hexs/Auto-Inspection-AI',
)



import time
from pathlib import Path

import hexss

hexss.check_packages(
    'numpy', 'opencv-python', 'Flask', 'requests', 'pygame', 'pygame-gui',
    'tensorflow', 'keras', 'pyzbar', 'AutoInspection', 'matplotlib',
    'flatbuffers==23.5.26',
    auto_install=True
)

import cv2
import numpy as np
from hexss import json_load, close_port, system, username
from hexss.server import camera_server
from hexss.image import Image
from AutoInspection import AutoInspection, training
from AutoInspection.server import run_server


def capture(data):
    clahe = cv2.createCLAHE(clipLimit=2.0, tileGridSize=(8, 8))

    def apply_clahe(bgr):
        lab = cv2.cvtColor(bgr, cv2.COLOR_BGR2LAB)
        L, A, B = cv2.split(lab)
        L = clahe.apply(L)
        lab = cv2.merge([L, A, B])
        return cv2.cvtColor(lab, cv2.COLOR_LAB2BGR)

    # ---- Option B: Local tone mapping (compress highlights, lift shadows) ----
    def tone_map_local(bgr, shadow=0.5, highlight=0.6, sigma=15):
        """
        shadow   : 0..1  (higher -> stronger lift for dark areas)
        highlight: 0..1  (higher -> stronger compression for bright areas)
        sigma    : Gaussian sigma for the base layer (larger -> more global)
        """
        f = bgr.astype(np.float32) / 255.0
        # luminance (BT.709)
        Y = 0.2126 * f[..., 2] + 0.7152 * f[..., 1] + 0.0722 * f[..., 0]
        base = cv2.GaussianBlur(Y, (0, 0), sigmaX=sigma, sigmaY=sigma)
        mid = 0.5
        eps = 1e-6

        gain = np.ones_like(base, dtype=np.float32)
        shadows_mask = base < mid
        highlights_mask = ~shadows_mask

        # lift shadows: bring base up toward 0.5
        gain[shadows_mask] = (mid / (base[shadows_mask] + eps)) ** shadow
        # compress highlights: bring base down toward 0.5
        gain[highlights_mask] = (mid / (base[highlights_mask] + eps)) ** highlight

        out = f * gain[..., None]
        out = np.clip(out, 0.0, 1.0)
        return (out * 255.0).astype(np.uint8)

    while data['play']:
        try:
            im1 = Image('http://127.0.0.1:2002/image?source=video_capture&id=0&quality=100')
            im2 = Image('http://127.0.0.1:2002/image?source=video_capture&id=1&quality=100')

            im1.rotate(-90, expand=True)
            im2.rotate(-90, expand=True)

            img1 = tone_map_local(im1.numpy(), shadow=0.75, highlight=0.75, sigma=20)
            img2 = tone_map_local(im2.numpy(), shadow=0.75, highlight=0.75, sigma=20)
            img = np.concatenate((img1, img2), axis=1)
        except:
            img = np.full([3264, 4896, 3], [50, 50, 50], np.uint8)
        data['img'] = img


def main(data):
    app = AutoInspection(data)
    app.run()


if __name__ == '__main__':
    from hexss.threading import Multithread

    config = json_load('config.json', {
        'projects_directory': r'C:\PythonProjects' if system == 'Windows' else f'/home/{username}/PythonProjects',
        'ipv4': '0.0.0.0',
        'port': 3000,
        'resolution_note': '1920x1080, 800x480',
        'resolution': '1920x1080' if system == 'Windows' else '800x480',
        'model_name': '-',
        'model_names': ["QC7-7990-000-Example", ],
        'fullscreen': True,
    }, True)

    close_port(config['ipv4'], config['port'], verbose=False)

    # download example
    if 'auto_inspection_data__QC7-7990-000-Example' not in \
            list(p.name for p in Path(config['projects_directory']).iterdir()):
        from hexss.github import download

        download(
            'hexs', 'auto_inspection_data__QC7-7990-000-Example',
            dest_dir=Path(config['projects_directory']) / 'auto_inspection_data__QC7-7990-000-Example'
        )

    # training
    try:
        training(
            *config['model_names'],
            config={
                'projects_directory': config['projects_directory'],
                'batch_size': 32,
                'img_height': 180,
                'img_width': 180,
                'epochs': 10,
                'shift_values': [-4, 0, 4],
                'brightness_values': [-12, 0, 12],
                'contrast_values': [-6, 0, 6],
                'max_file': 20000,
            }
        )
    except Exception as e:
        print(e)

    m = Multithread()
    data = {
        'config': config,
        'model_name': config['model_name'],
        'model_names': config['model_names'],
        'events': [],
        'play': True,

    }
    m.add_func(camera_server.run, join=False)
    m.add_func(capture, args=(data,))
    m.add_func(main, args=(data,))
    m.add_func(run_server, args=(data,), join=False)

    m.start()
    try:
        while data['play']:
            # print(m.get_status())
            time.sleep(1)
    except KeyboardInterrupt:
        print("\nShutting down...")
    finally:
        data['play'] = False
        m.join()



import subprocess

exe_path = r"C:\Users\c026730\AppData\Roaming\Python\Python312\Scripts\hexss.exe"

# รัน process และต่อ stdout/stderr กลับมาให้อ่าน
process = subprocess.Popen(
    [exe_path, "camera_server"],
    stdout=subprocess.PIPE,
    stderr=subprocess.STDOUT,
    text=True
)

print("hexss.exe started. Showing logs (Ctrl+C to stop):")

try:
    for line in process.stdout:
        print(line, end="")  # แสดง log แบบ realtime
except KeyboardInterrupt:
    print("\nStopped reading logs, but hexss.exe will stop when Python stops.")


import subprocess

exe_path = r"C:\Users\c026730\AppData\Roaming\Python\Python312\Scripts\hexss.exe"

DETACHED_PROCESS = 0x00000008
subprocess.Popen(
    [exe_path, "camera_server"],
    creationflags=DETACHED_PROCESS,
    stdout=subprocess.DEVNULL,
    stderr=subprocess.DEVNULL,
    close_fds=True
)

print("hexss.exe started and will keep running even if this script exits.")
